//
// surface.h
// Header for surface.cpp
//

#ifndef __GFXSCENE_H__
#define __GFXSCENE_H__


// Maximum images number in our project
//
#define MAX_IMAGES_COUNT 1024


// Forward declarations
//
class TIfaceElement;
class TGameElement;
class TScriptParser;
class TAutocreator;
class TMainMenu;
class TSoundSystem;


// Main class for whole game surface
//
class TSurface {

    public:

        // timer tick number 0..0xFFFFFFFF
        DWORD       tick_number;
        // last milliseconds (for fps counting)
        DWORD       last_msec_count;
        int         last_fps;

		// stylus coordinates
        int         stylus_down;
        int         stylus_last_x;
        int         stylus_last_y;

        // Game modes (normal playing, main menu and so on ...)
        int         game_mode;
		BOOL		is_paused;

        // Can we make graphics output or not?
        BOOL        graph_out_enabled;

        // Game vars
        int         Score;
		int			HighScore;

        // Currently loaded graphics
        TImage *        images[MAX_IMAGES_COUNT];

        // Sound system
        TSoundSystem    * sounds;

		// Console
		TCHAR			console_buffer[10][60];

        // Script parser
        TScriptParser   * script_parser;

		// Autocreator
		TAutocreator    * autocreator;

        // Main menu
        TMainMenu       * mainmenu;

		// Interface elements
        TIfaceElement   * iface_elements[IFC_MAX_ELEMENTS_COUNT];

		// Game elements
		TGameElement    * game_elements[GM_MAX_ELEMENTS_COUNT];


        TSurface();     // constructor
        ~TSurface();    // destructor

        void        DrawSurface ( void );
        void        OnMouseDown ( int mx, int my );
        void        OnMouseMove ( int mx, int my );
        void        OnMouseUp   ( int mx, int my );
        void        OnKeyDown   ( WPARAM vkey );
        void        TimerTick   ( void );

        BOOL        CollisionCheck ( int x, int y, int dx, int dy, int * retindex, int * retdx, int * retdy );

        void        ProcessGameKeys ( WPARAM vkey );
        void        ProcessMessage ( int msg, WPARAM wParam, LPARAM lParam );

        int         LocateFreeGE ( WORD area );

		void		ConsoleOutput ( LPCTSTR s );

        void        StartGame       ( void );
        void        StartLevel      ( int lev_num, BOOL is_continue );
};

// Class for interface elements (controls and so on)
//
class TIfaceElement {

    public:

        // It's screen coordinates
        int            x;
        int            y;
        int            dx;
        int            dy;

        bool           visible;
        bool           destroy_this;

        TSurface       * surface;

        TIfaceElement ( TSurface * sf ) { surface = sf; visible = true; destroy_this = FALSE; }
        ~TIfaceElement () {}

        virtual void Draw ( void ) {}

        virtual bool OnMouseDown ( int mx, int my ) { return false; }
        virtual void OnMouseMove ( int mx, int my ) {}
        virtual void OnMouseUp   ( int mx, int my ) {}

        virtual void TimerTick ( void ) {}
};

// Class for game elements (layout elements, obstacles, enemy and so...)
//
class TGameElement {

    public:

        TSurface       * surface;
        int            etype;          // type of descendant class (mainship, firing projectiles, enemy...)
		int			   esubtype;       // subtype for some elements

        BOOL           solid;          // solid object - will be hit by firing on it
        int            energy;         // hit points of this object
        int            weight;         // weight of object (1 - lightest object, 4 - mainship ... and so on)

        double         x;              // X coordinate
        double         y;              // Y

        double         inert_dx;       // inertion moment for collisions
        double         inert_dy;       //

        double         speed;          // speed for some objects

        int            imgnum;         // current image number
        double         sprnum;         // current sprite inside current image
        BOOL           is_highlight;   // is required highlighted drawing

        int            ai_mode;        // artificial intelligence mode

        bool           destroy_this;   // element will be disposed on first surface->TimerTick() if set to TRUE

        TGameElement ( TSurface * sf ) { 
            surface  = sf; 
            etype    = CLASS_UNKNOWN;
            esubtype = 0; 

            solid    = FALSE;
            energy   = 0xFFFF;
            weight   = 4;

            x = 100;
            y = 100;
            inert_dx = 0;
            inert_dy = 0;

            speed  = 1;
            imgnum = IMAGE_STARS;
            sprnum = 0;
            is_highlight = FALSE;

            ai_mode  = AI_MODE_NORMAL;

            destroy_this = FALSE;
        }
        ~TGameElement () {}

        virtual void Draw ( void ) {}
        virtual void TimerTick ( void ) {}

        virtual void AdjustInertion ( void ) {
            x += inert_dx;
            y += inert_dy;
            inert_dx = inert_dx / 1.5;
            inert_dy = inert_dy / 1.5;
        }

        virtual void IncSprNum ( double dt ) {
            sprnum += dt;
            if ((int)sprnum >= surface->images[imgnum]->count) sprnum = 0;
        }
};

// Script processor class
//
class TScriptParser {
    public:
        TSurface       * surface;
        CStringList    sdata;
        POSITION       curpos;

        int            current_command;
        CString        current_args[10];

		double		   wait_ticks;

        TScriptParser ( TSurface * sf );
        ~TScriptParser ();

        void    LoadFile   ( LPCTSTR fname );
        void    TimerTick  ( void );
        void    GetCommand ( void );
};

// Class for automatic creation of some objects (asteroids, clouds, etc..)
//
class TAutocreator {
    public:

        TSurface    * surface;
		WORD	    atype;

		double		aswarm_bgr1_wait;
		int			aswarm_bgr1_idx;
		double		aswarm_bgr2_wait;
		int			aswarm_bgr2_idx;
        double      aswarm_fore1_wait;
		int			aswarm_bgr1_off;
		int         aswarm_bgr2_off;
		int         aswarm_fore1_off;
		int         aswarm_all_off;

		double		sdust_up2_wait;

        TAutocreator ( TSurface * sf );
		~TAutocreator () {};
        void TimerTick ( void );

		void SetOn ( int __etype );
		void SetOff ( int __etype );
};

//////////////////////////////////////////////////////////////



// Delarations
//
extern  TSurface    * game_surface;
extern  double      GAME_SPEED_KOEFF;

extern  BOOL        InitSurface ( void );
extern  void        DeinitSurface ( void );



#endif // __GFXSCENE_H__
